use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007000
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
ConfigSlicer
overriden
makemaker
ini
PodWeaver
Taskweaver
debian
dh
Shantanu
Bhadoria
shantanu
https
Karen
Etheridge
ether
att
cpan
dott
org
lib
Badge
Depot
Plugin
Shantanutravis
Dist
Zilla
AutoMetaResourcesPrefixed
PluginBundle
SHANTANU
Pod
Weaver
